@@ -223,6 +223,7 @@ vips_foreign_save_ppm_build(VipsObject *object)
 	VipsImage *image;
 	char *magic;
 	char *date;
+	VipsBandFormat target_format;
 	VipsInterpretation target_interpretation;
 	int target_bands;
 
@@ -253,27 +254,37 @@ vips_foreign_save_ppm_build(VipsObject *object)
 	case VIPS_FOREIGN_PPM_FORMAT_PBM:
 		if (!vips_object_argument_isset(object, "bitdepth"))
 			ppm->bitdepth = 1;
+		target_format = VIPS_FORMAT_UCHAR;
 		target_interpretation = VIPS_INTERPRETATION_B_W;
 		target_bands = 1;
 		break;
 
 	case VIPS_FOREIGN_PPM_FORMAT_PGM:
-		if (image->BandFmt == VIPS_FORMAT_USHORT)
+		if (image->BandFmt == VIPS_FORMAT_USHORT) {
 			target_interpretation = VIPS_INTERPRETATION_GREY16;
-		else
+			target_format = VIPS_FORMAT_USHORT;
+		}
+		else {
 			target_interpretation = VIPS_INTERPRETATION_B_W;
+			target_format = VIPS_FORMAT_UCHAR;
+		}
 		target_bands = 1;
 		break;
 
 	case VIPS_FOREIGN_PPM_FORMAT_PPM:
-		if (image->BandFmt == VIPS_FORMAT_USHORT)
+		if (image->BandFmt == VIPS_FORMAT_USHORT) {
 			target_interpretation = VIPS_INTERPRETATION_RGB16;
-		else
+			target_format = VIPS_FORMAT_USHORT;
+		}
+		else {
 			target_interpretation = VIPS_INTERPRETATION_sRGB;
+			target_format = VIPS_FORMAT_UCHAR;
+		}
 		target_bands = 3;
 		break;
 
 	case VIPS_FOREIGN_PPM_FORMAT_PFM:
+		target_format = VIPS_FORMAT_FLOAT;
 		target_interpretation = VIPS_INTERPRETATION_scRGB;
 		if (image->Bands > 1)
 			target_bands = 3;
@@ -284,14 +295,19 @@ vips_foreign_save_ppm_build(VipsObject *object)
 
 	case VIPS_FOREIGN_PPM_FORMAT_PNM:
 	default:
-		/* Just use the input interpretation and bands.
+		/* Just use the input format, interpretation and bands.
 		 */
+		target_format = image->BandFmt;
 		target_interpretation = image->Type;
 		target_bands = image->Bands;
 		break;
 	}
 
-	if (vips_colourspace(image, &t[1], target_interpretation, NULL))
+	if (vips_colourspace_issupported(image)) {
+		if (vips_colourspace(image, &t[1], target_interpretation, NULL))
+			return -1;
+	}
+	else if (vips_cast(image, &t[1], target_format, NULL))
 		return -1;
 	image = t[1];
 
