@@ -185,7 +185,44 @@ vips_value_hash(GParamSpec *pspec, const GValue *value)
 	else if (generic == G_TYPE_PARAM_BOXED) {
 		void *p = g_value_get_boxed(value);
 
-		return p ? g_direct_hash(p) : 0;
+		// array_object is an internal type, don't have a case for this
+		if (!p)
+			return 0;
+		else if (G_VALUE_TYPE(value) == VIPS_TYPE_ARRAY_INT) {
+			int n;
+			int *array = (int *)
+				vips_area_get_data(VIPS_AREA(p), NULL, &n, NULL, NULL);
+
+			guint hash = 0;
+			for (int i = 0; i < n; i++)
+				hash = (hash << 1) ^ g_int_hash(&array[i]);
+
+			return hash;
+		}
+		else if (G_VALUE_TYPE(value) == VIPS_TYPE_ARRAY_DOUBLE) {
+			int n;
+			double *array = (double *)
+				vips_area_get_data(VIPS_AREA(p), NULL, &n, NULL, NULL);
+
+			guint hash = 0;
+			for (int i = 0; i < n; i++)
+				hash = (hash << 1) ^ g_double_hash(&array[i]);
+
+			return hash;
+		}
+		else if (G_VALUE_TYPE(value) == VIPS_TYPE_ARRAY_IMAGE) {
+			int n;
+			void **array = (void **)
+				vips_area_get_data(VIPS_AREA(p), NULL, &n, NULL, NULL);
+
+			guint hash = 0;
+			for (int i = 0; i < n; i++)
+				hash = (hash << 1) ^ g_direct_hash(array[i]);
+
+			return hash;
+		}
+		else
+			return g_direct_hash(p);
 	}
 	else if (generic == G_TYPE_PARAM_POINTER) {
 		void *p = g_value_get_pointer(value);
@@ -275,8 +312,77 @@ vips_value_equal(GParamSpec *pspec, const GValue *v1, const GValue *v2)
 		else
 			return s1 && s2 && strcmp(s1, s2) == 0;
 	}
-	if (generic == G_TYPE_PARAM_BOXED)
-		return g_value_get_boxed(v1) == g_value_get_boxed(v2);
+	if (generic == G_TYPE_PARAM_BOXED) {
+		void *p1 = g_value_get_boxed(v1);
+		void *p2 = g_value_get_boxed(v2);
+
+		if (p1 == p2)
+			return TRUE;
+		else if (!p1 || !p2)
+			return FALSE;
+		else if (t1 == VIPS_TYPE_ARRAY_INT) {
+			int n1;
+			int *array1 = (int *)
+				vips_area_get_data(VIPS_AREA(p1), NULL, &n1, NULL, NULL);
+			int n2;
+			int *array2 = (int *)
+				vips_area_get_data(VIPS_AREA(p2), NULL, &n2, NULL, NULL);
+
+			if (n1 != n2)
+				return FALSE;
+			else if (array1 == array2)
+				return TRUE;
+			else {
+				for (int i = 0; i < n1; i++)
+					if (array1[i] != array2[i])
+						return FALSE;
+
+				return TRUE;
+			}
+		}
+		else if (t1 == VIPS_TYPE_ARRAY_DOUBLE) {
+			int n1;
+			double *array1 = (double *)
+				vips_area_get_data(VIPS_AREA(p1), NULL, &n1, NULL, NULL);
+			int n2;
+			double *array2 = (double *)
+				vips_area_get_data(VIPS_AREA(p2), NULL, &n2, NULL, NULL);
+
+			if (n1 != n2)
+				return FALSE;
+			else if (array1 == array2)
+				return TRUE;
+			else {
+				for (int i = 0; i < n1; i++)
+					if (array1[i] != array2[i])
+						return FALSE;
+
+				return TRUE;
+			}
+		}
+		else if (t1 == VIPS_TYPE_ARRAY_IMAGE) {
+			int n1;
+			void **array1 = (void **)
+				vips_area_get_data(VIPS_AREA(p1), NULL, &n1, NULL, NULL);
+			int n2;
+			void **array2 = (void **)
+				vips_area_get_data(VIPS_AREA(p2), NULL, &n2, NULL, NULL);
+
+			if (n1 != n2)
+				return FALSE;
+			else if (array1 == array2)
+				return TRUE;
+			else {
+				for (int i = 0; i < n1; i++)
+					if (array1[i] != array2[i])
+						return FALSE;
+
+				return TRUE;
+			}
+		}
+		else
+			return p1 == p2;
+	}
 	if (generic == G_TYPE_PARAM_POINTER)
 		return g_value_get_pointer(v1) == g_value_get_pointer(v2);
 	if (generic == G_TYPE_PARAM_OBJECT)
@@ -815,6 +921,49 @@ vips_cache_trim(void)
 	g_mutex_unlock(&vips_cache_lock);
 }
 
+#ifdef DEBUG_LEAK
+static void *
+vips_cache_find_differences(VipsObject *object,
+	GParamSpec *pspec,
+	VipsArgumentClass *argument_class,
+	VipsArgumentInstance *argument_instance,
+	void *a, void *b)
+{
+	VipsOperation *operation_before = VIPS_OPERATION(a);
+
+	if ((argument_class->flags & VIPS_ARGUMENT_CONSTRUCT) &&
+		(argument_class->flags & VIPS_ARGUMENT_INPUT) &&
+		argument_instance->assigned) {
+		const char *name = g_param_spec_get_name(pspec);
+
+		GValue value_before = G_VALUE_INIT;
+		g_object_get_property(G_OBJECT(operation_before), name, &value_before);
+		unsigned int hash_before = vips_value_hash(pspec, &value_before);
+
+		GValue value_after = G_VALUE_INIT;
+		g_object_get_property(G_OBJECT(object), name, &value_after);
+		unsigned int hash_after = vips_value_hash(pspec, &value_after);
+
+		if (hash_before != hash_after) {
+			g_warning("arg \"%s\" has changed value during build", name);
+
+			char *str_before = g_strdup_value_contents(&value_before);
+			g_warning("\tvalue before: %s", str_before);
+			g_free(str_before);
+
+			char *str_after = g_strdup_value_contents(&value_after);
+			g_warning("\tvalue after: %s", str_after);
+			g_free(str_after);
+		}
+
+		g_value_unset(&value_before);
+		g_value_unset(&value_after);
+	}
+
+	return NULL;
+}
+#endif /*DEBUG_LEAK*/
+
 /**
  * vips_cache_operation_buildp: (skip)
  * @operation: pointer to operation to lookup
@@ -901,6 +1050,11 @@ vips_cache_operation_buildp(VipsOperation **operation)
 		 */
 		if (vips__leak) {
 			hash_before = vips_operation_hash(*operation);
+
+			/* This isn't a deep copy, so it won't detect eg.
+			 * operations modifying compound objects like VipsArrayInt.
+			 * However, the hash_before value we save will.
+			 */
 			operation_before = vips_operation_copy(*operation);
 		}
 #endif /*DEBUG_LEAK*/
@@ -912,25 +1066,14 @@ vips_cache_operation_buildp(VipsOperation **operation)
 		if (vips__leak &&
 			!(flags & VIPS_OPERATION_NOCACHE) &&
 			hash_before != vips_operation_hash(*operation)) {
-			const char *name = (const char *)
-				vips_argument_map(VIPS_OBJECT(*operation),
-					vips_object_equal_arg, operation_before, NULL);
-			VipsObject *object = VIPS_OBJECT(*operation);
-			VipsObjectClass *class = VIPS_OBJECT_GET_CLASS(object);
-
 			char txt[256];
 			VipsBuf buf = VIPS_BUF_STATIC(txt);
+			vips_object_summary(VIPS_OBJECT(operation_before), &buf);
+			g_warning("vips_cache_operation_buildp: "
+				"arg mismatch on build of: %s", vips_buf_all(&buf));
 
-			VIPS_UNREF(operation_before);
-
-			vips_object_summary_class(class, &buf);
-			vips_buf_appends(&buf, ", ");
-			vips_object_summary(object, &buf);
-			vips_buf_appends(&buf, ", ");
-			vips_error(class->nickname, "arg \"%s\" changed during build, %s",
-				name, vips_buf_all(&buf));
-
-			return -1;
+			vips_argument_map(VIPS_OBJECT(*operation),
+				vips_cache_find_differences, operation_before, NULL);
 		}
 
 		VIPS_UNREF(operation_before);
